/*
   _===_ _           _                  ______ _     _             _ _ _
  |  ___| |         | |                 | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|

  Contrôleur Amélioré pour Orienteur d'Antenne HyGain TailTwister (ou similaire).
  Utilise un Raspberri Pi Pico.
  Électro-Bidouilleur, Novembre 2022. https://www.youtube.com/Électro-Bidouilleur
  Licence de droits: Attribution 4.0 International (CC BY 4.0)  Lisez https://creativecommons.org/licenses/by/4.0/deed.fr  
*/

// DIRECTIVES AU COMPILATEUR

#include "Adafruit_ILI9341.h"
#include "Adafruit_GFX.h"
#include <XPT2046_Touchscreen.h>

 // Voici les connexions sur un Rasprerry Pi Pico. Les numéros indiqués correspondent aux appellations GPxx du brochage.
#define TFT_DC 21
#define TFT_CS 17
#define TFT_MOSI 19
#define TFT_CLK 18
#define TFT_RST 20
#define TFT_MISO 16
#define TCS_PIN  13
#define TIRQ_PIN 14

#define Relai_Frein 10
#define Relai_Rot_H 11
#define Relai_Rot_AH 12
#define Pot_Curseur 26
#define Pot_Max 27
#define Pot_Min 28
#define encod_a 2
#define encod_b 3
#define encod_bout_pouss 4
#define control_eclair_affich 5

#define hysteresis_orient 5
#define ralentissement_post_orient 2      // Valeur de la rotation résiduelle en degrés, après coupure du moteur
#define num_echant_pot 200

#define centre_cercle_x 239
#define centre_cercle_y 81
#define rayon_cercle 55

#define couleur_en_arret ILI9341_GREEN
#define couleur_en_rotation ILI9341_MAGENTA
#define couleur_en_ralentissement ILI9341_YELLOW


// INSTANCIATION DES CLASSES
// Utilisation du port SPI intégré (port matériel) du STM32 Blackpill. Il faut spécifier la position des broches CS, DC et RST pour l'affichage.
Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC,TFT_RST); //

// Implémentation de la lecture tactile en utilisant la broche IRQ
XPT2046_Touchscreen ts(TCS_PIN, TIRQ_PIN);  // Param 2 - Touch IRQ Pin - interrupt enabled polling


// VARIABLES GLOBALES
int orient_courante;
int orient_instant;
int orient_cible = -1;
int orient_preced_verif = 0;
int orient_preced_affich = 0;
unsigned long int debut_ralentissement_en_cours_millis = 0; 
unsigned long int debut_verif_rotation_millis = 0; 
unsigned long int debut_rafraich_ecran_millis = 0;
boolean verif_rotation_en_cours = false;
boolean appel_rafraich_azim = true;
boolean alarme_active = false;
boolean sens_h = false;
String nombre = "";
String commande = "";
double moy_px = 0;
double moy_py = 0;
int moy_compte = 0;
boolean orient_en_cours = false;
boolean ralentissement_en_cours = false;
boolean arret_en_cours = true;
boolean appel_orient = false;
boolean appel_ralentissement = false;
boolean appel_arret = true;
boolean etat_encodeur;
boolean etat_preced_encodeur;
unsigned long int debut_compte_affich_millis = 0;
boolean orient_dispo = false;


// Fonction de conversion texte et ajout de caractères "0" en amont de la représentation du nombre. Pour fins de formatage texte
String int_bourre_zeros_str(int a)
{
  if (a >= 10 && a < 100) return "0" + String(a);         // Ajouter un 0 si le nombre est entre 10 et 100 et retorner la conversion texte
  else if (a < 10) return "00" + String(a);               // Ajouter deux 0 si le nombre est entre 1 et 10 et retorner la conversion texte
  else return String(a);                                  // Sinon ne rajouter aucun zéros et retorner la conversion texte
}


// Fonction d'initialisation du premier coeur du Pico, style Arduino, exécutée une seule fois à la mise sous tension
void setup() 
{
  pinMode(TCS_PIN,OUTPUT);                                // S'assurer que les broches CS sont au niveau haut pour commencer
  digitalWrite(TCS_PIN,HIGH);                             //    "
  pinMode(TFT_CS,OUTPUT);                                 //    "
  digitalWrite(TFT_CS,HIGH);                              //    "
  pinMode(Relai_Frein,OUTPUT);                            // Même chose pour les broches de relais 
  digitalWrite(Relai_Frein,HIGH);                         //    "
  pinMode(Relai_Rot_H,OUTPUT);                            //    "
  digitalWrite(Relai_Rot_H,HIGH);                         //    "
  pinMode(Relai_Rot_AH,OUTPUT);                           //    " 
  digitalWrite(Relai_Rot_AH,HIGH);                        //    "
  pinMode(encod_a,INPUT_PULLUP);                          // Initialiser les broches d'encodeur comme entrées
  pinMode(encod_b,INPUT_PULLUP);                          //    "
  pinMode(encod_bout_pouss,INPUT_PULLUP);                 //    "
  pinMode(control_eclair_affich,OUTPUT);                  // Initialiser la broche de contrôle d'éclairage de l'affichage comme sortie et allumer l'éclairage
  digitalWrite(control_eclair_affich,HIGH);               //    "
 
//  Serial.begin(9600);                                   // Activer seulement pour déboguer
  tft.begin(24000000);                                    // Démarrer l'instance d'écran
  tft.setRotation(3);                                     // Faire pivoter l'affichage de 270 degrés
  ts.begin();                                             // Démarrer l'instance de détection tactile

  while (!Serial && (millis() <= 1000));                  // Une pause d'au moins 1 seconde
  while (!orient_dispo);                                  // Attendre que l'orientation courante ait été lue par le deuxième coeur
  orient_courante = orient_instant;                       // Initialiser les variables d'orientation courante
  orient_preced_affich = orient_courante;                 //    "
  orient_dispo = false;                                   //    "

  //Dessin de la boussole
  tft.fillScreen(ILI9341_BLACK);    
  tft.drawCircle(centre_cercle_x, centre_cercle_y, rayon_cercle, ILI9341_YELLOW);
  tft.drawLine(centre_cercle_x, centre_cercle_y, centre_cercle_x + (rayon_cercle-1) * sin(float(orient_courante)/180*3.1416), centre_cercle_y - (rayon_cercle-1) * cos(float(orient_courante)/180*3.1416), couleur_en_arret);
  tft.setTextSize(2);      
  tft.setTextColor(ILI9341_WHITE);
  tft.setCursor(centre_cercle_x-5, centre_cercle_y-rayon_cercle-18);
  tft.print("N");
  tft.setCursor(centre_cercle_x-5, centre_cercle_y+rayon_cercle+6);
  tft.print("S");
  tft.setCursor(centre_cercle_x-rayon_cercle-15, centre_cercle_y-7);
  tft.print("O");
  tft.setCursor(centre_cercle_x+rayon_cercle+6, centre_cercle_y-7);
  tft.print("E");
  for (int i=0;i<360;i+=30)     // Dessin des lignes de graduation autour du cercle
  {
    tft.drawLine(centre_cercle_x + (rayon_cercle) * sin(float(i)/180*3.1416), centre_cercle_y - (rayon_cercle) * cos(float(i)/180*3.1416), centre_cercle_x + (rayon_cercle+5) * sin(float(i)/180*3.1416), centre_cercle_y - (rayon_cercle+5) * cos(float(i)/180*3.1416), ILI9341_WHITE);
  }

  // Dessin du clavier 
  tft.setTextSize(3);      
  tft.setTextColor(ILI9341_WHITE);
  for (int i = 16; i <= 160;i+=36)                          // Dessin des boutons bleus
  {
    tft.fillRect(i-5, 162, 33, 33, ILI9341_BLUE);
    tft.fillRect(i-5, 199, 33, 33, ILI9341_BLUE);
  }
  for (int i = 0; i <= 4;i++)                               // Dessin des chiffres de 0 à 9
  {
    tft.setCursor(20 + i*36, 169);
    tft.print(i);
    tft.setCursor(20 + i*36, 205);
    tft.print(i+5);
  }
  for (int i = 196; i <= 260;i+=62)                         // Dessin des autres textes de boutons
  {                                                         //      "
    tft.fillRect(i-5, 162, 59, 33, ILI9341_BLUE);
    tft.fillRect(i-5, 199, 59, 33, ILI9341_BLUE);
  }
  tft.setTextColor(ILI9341_GREEN);
  tft.setCursor(203, 169);
  tft.print("GO");
  tft.setTextColor(ILI9341_ORANGE);
  tft.setCursor(195, 205);
  tft.print("CLR");
  tft.setTextColor(ILI9341_WHITE);
  tft.setCursor(256, 169);
  tft.print("+15");
  tft.setCursor(256, 205);
  tft.print("-15");

  tft.setTextColor(couleur_en_arret);                        // Dessin du texte de statut courant de l'orienteur
  appel_rafraich_azim = true;
  etat_preced_encodeur = digitalRead(encod_a);               // Lecture du niveau logique du signal A de l'encodeur
}


// Fonction d'initialisation du deuxième coeur du Pico, style Arduino, exécutée une seule fois à la mise sous tension
void setup1() 
{
  pinMode(Pot_Curseur,INPUT);                                // Configurer les broches d'ADC en entrées 
  pinMode(Pot_Max,INPUT);                                    //     "
  pinMode(Pot_Min,INPUT);                                    //     "
  analogReadResolution(12);                                  // Configurer la résolution de l'ADC à 12 bits
}


// La boucle principale, style Arduino. Elle tourne de façon perpétuelle sur le premier coeur du Pico.
void loop() 
{
  if (orient_dispo)                                                                   // Nouvelle orientation courante disponible?
  {
    orient_courante = orient_instant;                                                 // Mettre à jour l'orientation instantanée.
    orient_dispo = false;
  }

  // Rafraîchissement de l'affichage
  if ((millis() - debut_rafraich_ecran_millis) >= 100)                                // Rafraîchissement à toutes les 100 millisecondes
  {  
    debut_rafraich_ecran_millis = millis();                                           // Initialiser un nouveau cycle de 100 millisecondes
    if ((orient_preced_affich != orient_courante) || (appel_rafraich_azim))           // Mise à jour de l'orientation courante sur l'affichage si nécessaire
    {
      String orient_courante_txt = int_bourre_zeros_str(orient_courante);             // Ajouter des zéros pour faire trois caractères si nécessaire
      String orient_preced_affich_txt = int_bourre_zeros_str(orient_preced_affich);   // Sauvegarde de l'orientation courante pour le prochain cycle d'affichage
      for (int i=0;i<=2;i++)                                                          // Mise à jour de l'affichage des trois caractères de l'orientation 
      {
        if ((orient_courante_txt[i] != orient_preced_affich_txt[i]) || (appel_rafraich_azim))   // Mise à jour d'un caractère seulement si requis (évite les clignotements)
        {
          tft.setTextSize(8);      
          tft.setCursor(15 + 50*i, 20);                                                     // Effacement de l'ancien caractère (en noir)
          tft.setTextColor(ILI9341_BLACK);                                                  //      "
          tft.print(orient_preced_affich_txt[i]);                                           //      "
          tft.setCursor(15 + 50*i, 20);                                                     // Écriture du nouveau caractère (avec la bonne couleur en fonction de l'action courante)  
          if (orient_en_cours) tft.setTextColor(couleur_en_rotation);                       //      "
          else if (ralentissement_en_cours) tft.setTextColor(couleur_en_ralentissement);    //      "
          else tft.setTextColor(couleur_en_arret);                                          //      "
          tft.print(orient_courante_txt[i]);                                                //      "
        }  
      }
      // Mise à jour de la boussole
      tft.drawCircle(centre_cercle_x, centre_cercle_y, rayon_cercle, ILI9341_YELLOW);     
      tft.drawLine(centre_cercle_x, centre_cercle_y, centre_cercle_x + (rayon_cercle-1) * sin(float(orient_preced_affich)/180*3.1416), centre_cercle_y - (rayon_cercle-1) * cos(float(orient_preced_affich)/180*3.1416), ILI9341_BLACK);    // Effacement de l'ancienne aiguille
      tft.drawLine(centre_cercle_x, centre_cercle_y, centre_cercle_x + (rayon_cercle-1) * sin(float(orient_courante)/180*3.1416), centre_cercle_y - (rayon_cercle-1) * cos(float(orient_courante)/180*3.1416), (orient_en_cours || ralentissement_en_cours ? couleur_en_rotation : couleur_en_arret));  // Dessin de la nouvelle aiguille avec la bonne couleur   
    }
    if (alarme_active && orient_en_cours)                                 // Effacement du message d'alarme?
    {
      tft.fillRect(20, 120, 130, 40, ILI9341_BLACK);                      // Effacement du message d'alarme
      alarme_active = false;                                              // Effacement de l'alarme
    }
    appel_rafraich_azim = false;                                          // Abaisser le fanion d'appel de rafraîchissement d'affichage
    orient_preced_affich = orient_courante;              
  }

  // Traitement des actions tactiles
  if (ts.touched())                                                 // Affichage touché?
  {                                                                 // Oui
    if (!digitalRead(control_eclair_affich))                        // Affichage éteint?
    {                                                               // Oui
      debut_compte_affich_millis = millis();                        // Réinitialiser la minuterie d'éteignement de l'affichage
      digitalWrite(control_eclair_affich,HIGH);                     // Allumer l'affichage 
      while (ts.touched());                                         // Attendre le relâchement de la touche
    }
    alarme_active = false;                                          // Abaisser le fanion d'alarme active
    while (ts.touched())                                            // Détection de la touche ?
    {                                                               // Oui
      TS_Point p = ts.getPoint();                                   // Obtenir les coordonnées de touche     
      moy_px += p.x;                                                // Calcul de la moyenne des coordonnées (augmente la précision)
      moy_py += p.y;                                                //     "
      moy_compte++;                                                 //     "      
    }
    delay(100);                                                     // Nécessaire pour l'anti-rebond tactile.
    moy_px = round(float(moy_px)/moy_compte);                       // Calcul de la moyenne des coordonnées (augmente la précision)
    moy_py = round(float(moy_py)/moy_compte);                       //     "

    // Décodage du bouton sélectionné en fonction des coordonnées
    if (moy_py > 2700 && moy_py <= 3250)                            // Décodage de la ligne de boutons
    {
      if      (moy_px > 150 && moy_px <= 600)   nombre += "0";      // Décodage de la colonne de boutons
      else if (moy_px > 600 && moy_px <= 1050)  nombre += "1";      //     "
      else if (moy_px > 1050 && moy_px <= 1500) nombre += "2";
      else if (moy_px > 1500 && moy_px <= 1950) nombre += "3";
      else if (moy_px > 1950 && moy_px <= 2400) nombre += "4";
      else if (moy_px > 2400 && moy_px <= 3100) commande = "GO";
      else if (moy_px > 3100 && moy_px <= 3900) commande = "+15";
    }
    else if (moy_py > 3250 && moy_py <= 3800)                       // Décodage de la ligne de boutons
    {
      if      (moy_px > 150 && moy_px <= 600)   nombre += "5";      // Décodage de la colonne de boutons
      else if (moy_px > 600 && moy_px <= 1050)  nombre += "6";      //     "
      else if (moy_px > 1050 && moy_px <= 1500) nombre += "7";
      else if (moy_px > 1500 && moy_px <= 1950) nombre += "8";
      else if (moy_px > 1950 && moy_px <= 2400) nombre += "9";
      else if (moy_px > 2400 && moy_px <= 3100) commande = "CLR";
      else if (moy_px > 3100 && moy_px <= 3900) commande = "-15";
    }
    moy_px = 0;                                                         
    moy_py = 0;
    moy_compte = 0;
    tft.fillRect(20, 120, 130, 40, ILI9341_BLACK);                  // Effacement de l'ancien caractère
    tft.setTextColor(ILI9341_GREEN);                                // Écriture du caractère du bouton décodé
    tft.setTextSize(4);                                             //     "
    tft.setCursor(20,120);                                          //     "
    tft.print(nombre);                                              //     "
    if ((nombre.length() > 3) || (nombre.toInt() > 360)) commande = "CLR";     // Effacement des caractère de bouton décodés si la nouvelle orientation est non-valide
  
    if (commande == "GO")                                           // Si bouton GO décodé
    {
      if (nombre.length() != 0)                                     // Si une nouvelle orientation est disponible
      {
        orient_cible = nombre.toInt();                              // Sauvegarder la nouvelle orientation en orientation cible pour être traitée
        appel_orient = true;                                        // Lancer une orientation
        nombre = "";
        commande = "";
      }
    }
    else if (commande == "CLR")                                     // Si bouton CLR décodé
    {
      if ((!orient_en_cours) && (!ralentissement_en_cours))         // S'il n'y a pas d'orientation en cours 
      {
        tft.fillRect(20, 120, 130, 40, ILI9341_BLACK);              // Effacer la nouvelle orientation et les variables
        nombre = "";                                                //     "
        commande = "";
        orient_cible = -1;
      }
      else if (orient_en_cours)                                     // S'il y a une orientation en cours
      {
        appel_ralentissement = true;                                // Appeler l'interruption de l'orientation en cours
        verif_rotation_en_cours = false;
      }
    }
    else if (((commande == "+15") || (commande == "-15")) && (!orient_en_cours) && (!ralentissement_en_cours))    // Si bouton +15 ou -15 décodé
    {
      orient_cible = orient_courante + commande.toInt();            // Augmenter ou diminuer l'orientation cible par 15
      if (orient_cible > 360) orient_cible = 360;                   // Ne pas dépasser 360 degrés
      else if (orient_cible < 0) orient_cible = 0;                  // Ne pas permettre une orientation négative
      tft.fillRect(20, 90, 130, 40, ILI9341_BLACK);                 // Effacer l'ancienne orientation cible
      tft.setTextColor(ILI9341_GREEN);                              // Écrire la nouvelle orientation cible
      tft.setTextSize(3);                                           //      "
      tft.setCursor(20,120);                                        //      "
      tft.print(String(orient_cible));                              //      "        
      appel_orient = true;                                          // Lancer une nouvelle orientation
      nombre = "";
      commande = "";
    }
  }

  // Traitement des rotations de l'encodeur
  if ((!orient_en_cours) && (!ralentissement_en_cours))       // Si pas d'orientation ou de ralentissement en cours
  { 
    etat_encodeur = digitalRead(encod_a);                     // Lecture du niveau logique du signal A de l'encodeur
    if (etat_encodeur != etat_preced_encodeur)                // nouveau niveau logique sur A?
    {   // Oui
      if (orient_cible == -1) orient_cible = orient_courante; // S'il n'y a pas déjà d'orientation cible, l'initialiser à l'orientation courante
      if (digitalRead(encod_b) != etat_encodeur)              // Lecture du niveau logique du signal B de l'encodeur. Même niveau que A?
      {  // Non   
        orient_cible += 5;                                    // Augmenter l'orientation cible de 5 degrés
        while (orient_cible%5 != 0) orient_cible--;           // Arrondir au nombre finissant par 5 le plus proche
        if (orient_cible > 360) orient_cible = 360;           // S'assurer que l'orientation ne dépasse pas 360 degrés
      } 
      else                                                    // Niveau logique du signal B de l'encodeur différent de A.
      {
        orient_cible -= 5;                                    // Diminuer l'orientation cible de 5 degrés
        while (orient_cible%5 != 0) orient_cible++;           // Arrondir au nombre finissant par 5 le plus proche
        if (orient_cible < 0) orient_cible = 0;               // S'assurer que l'orientation ne va pas en deça de 0 degrés
      }
      tft.fillRect(20, 120, 130, 40, ILI9341_BLACK);          // Mettre à jour l'orientation cible à l'écran
      tft.setTextColor(ILI9341_GREEN);                        //      "
      tft.setTextSize(4);                                     //      "
      tft.setCursor(20,120);                                  //      "
      tft.print(orient_cible);                                //      "
    } 
    etat_preced_encodeur = etat_encodeur;                     // Sauvegarder la valeur du signal A pour le prochain passage
  }
  
  // Détection du début d'une orientation via le bouton poussoir ou d'un appel du clavier
  if (((!orient_en_cours) && (!digitalRead(encod_bout_pouss)) && (orient_cible != -1)) || (appel_orient))  // Les conditions à rencontrer pour effectuer une rotation
    {
      orient_en_cours = true;                                 // Lever le fanion d'une orientation en cours
      appel_rafraich_azim = true;                             // Faire une demande au rafraîchissement de l'azimuth
      digitalWrite(Relai_Frein,LOW);                          // Relâcher le frein (activer le solénoïde)
      appel_orient = true;                                    // Faire une demande d'orientation
      delay(500);                                             // Faire une pause
    }

  // Traitement des messages de statut affichés
  if (appel_orient)                                           // Demande d'orientation?
  {  // oui
    tft.fillRect(20, 90, 130, 30, ILI9341_BLACK);             // Afficher le message de rotation
    tft.setTextSize(2);                                       //               "
    tft.setTextColor(couleur_en_rotation);
    tft.setCursor(20,90);
    tft.print("EN ROTATION");
  }
  else if (appel_ralentissement)                              // Sinon, demande de ralentissement?
  { // Oui
    tft.fillRect(20, 90, 130, 30, ILI9341_BLACK);             // Afficher le message de ralentissement
    tft.setTextSize(2);                                       //               "      
    tft.setTextColor(couleur_en_ralentissement);
    tft.setCursor(20,90);
    tft.print("EN INERTIE");
    appel_ralentissement = false;                             // Annuler l'appel de ralentissement
  }
  else if (appel_arret)                                       // Sinon, demande d'arrêt?
  {  // Oui
    tft.fillRect(20, 90, 130, 30, ILI9341_BLACK);             // Afficher le message "en attente"
    tft.setTextSize(2);                                       //               "      
    tft.setTextColor(couleur_en_arret);
    tft.setCursor(20,90);
    tft.print("EN ATTENTE");
    appel_arret = false;                                      // Annuler l'appel d'arrêt
  }

  // traitement du processus d'orientation
  if (orient_en_cours)                                        // Processus d'orientation en cours?
  { // Oui
    if (appel_orient)                                         // Demande d'orientation?
    {  // Oui
      if ((orient_courante + ralentissement_post_orient) < orient_cible)      // Processus d'orientation dans le sens horaire
      {
        digitalWrite(Relai_Rot_AH,HIGH);                      // Désactiver le relai de rotation sens anti-horaire
        digitalWrite(Relai_Rot_H,LOW);                        // Activer le relai de rotation sens horaire                
        sens_h = true;                                        // Indiquer le sens de la rotation horaire
      }
      else if ((orient_courante - ralentissement_post_orient) > orient_cible) // Sinon, processus d'orientation dans le sens anti-horaire
      { // Oui
        digitalWrite(Relai_Rot_H,HIGH);                       // Désactiver le relai de rotation sens horaire
        digitalWrite(Relai_Rot_AH,LOW);                       // Activer le relai de rotation sens anti-horaire          
        sens_h = false;                                       // Indiquer le sens de la rotation anti-horaire
      }
      appel_orient = false;                                   // effacer le fanion de demande d'orientation
    }
    else
    {  // Déjà en rotation    
      if ((!digitalRead(encod_bout_pouss))                                                   // arrêt forcé via bouton poussoir ?
          || ((sens_h)  && ((orient_courante + ralentissement_post_orient) >= orient_cible)) // ou position atteinte (ou dépassée) en rotation horaire ?
          || ((!sens_h) && ((orient_courante - ralentissement_post_orient) <= orient_cible)) // ou position atteinte (ou dépassée) en rotation anti-horaire ?
          || (appel_ralentissement))                                                         // ou demande de ralentissement?
      {
        ralentissement_en_cours = true;                       // Lever le fanion de ralentissement en cours
        appel_ralentissement = true;                          // Faire une demande de ralentissement
        appel_rafraich_azim = true;                           // Faire une demande de rafraîchissement de l'azimuth  
      } 
    }
     // Autes actions durant la phase de rotation (pas en ralentissement)
    if (!ralentissement_en_cours)                             // Pas de ralentissement en cours 6
    {
      // Vérification que le rotor tourne bel et bien
      if (!verif_rotation_en_cours)                           // Pas en vérification de la rotation?                     
      { // Début d'un cycle d'attente avant vérification de rotation
        debut_verif_rotation_millis = millis();               // Sauvegarder le temps du début du délai de la vérification de rotation
        orient_preced_verif = orient_courante;                // Sauvegarder l'orientation courante comme orientation précédente
        verif_rotation_en_cours = true;                       // Lever le fanion de rotation en cours
      }
      else if ((millis() - debut_verif_rotation_millis) >= 2000)    // Si l'attente de vérification de rotation est écoulée, faire la vérification
      {
        verif_rotation_en_cours = false;                      // Marquer la fin du cycle de vérification de la rotation
        if ((orient_courante >= ((orient_preced_verif - 2 < 0) ? 0 : orient_preced_verif - 2 )) 
           && (orient_courante <= ((orient_preced_verif + 2 > 360) ? 360 : orient_preced_verif + 2)))  // Roror a tourné d'au moins 2 degrés ?
        { // Non. Rotor figé. Arrêter tout
          appel_ralentissement = true;                        // Faire une demande de ralentissement 
          ralentissement_en_cours = true;                     // Lever le fanion de ralentissement en cours
          tft.fillRect(20, 120, 130, 40, ILI9341_BLACK);      // Afficher le message d'alarme de rotation
          tft.setTextSize(2);                                 //             "
          tft.setTextColor(ILI9341_RED);
          tft.setCursor(20,120);
          tft.print("ALARME");
          tft.setCursor(20,140);
          tft.print("ROTATION");
          alarme_active = true;                               // Soulever le fanion d'alarme active
        }     
      }
    }
  }
  // Traiter la phase de ralentissement (fin de rotation)
  if (ralentissement_en_cours)                                              // Ralentissement en cours?
  {
    if (appel_ralentissement)    // Début de la phase de ralentissement     // Est-ce un nouvel appel au ralentissement?
    {
      digitalWrite(Relai_Rot_H,HIGH);                                       // Stopper la rotation (relâcher le relai)
      digitalWrite(Relai_Rot_AH,HIGH);                                      //     "          
      orient_en_cours = false;                                              // Terminer la rotation
      appel_ralentissement = true;                                          // Faire un appel au ralentissement
      appel_rafraich_azim = true;                                           // Faire un appel au rafraîchissement de l'affichage
      debut_ralentissement_en_cours_millis = millis();                      // Initialiser la minuterie de ralentissement
    }  
    else if ((millis() - debut_ralentissement_en_cours_millis) >= 6000)     // Temps de ralentissement écoulé?
    {
      ralentissement_en_cours = false;                                      // Oui, terminer la phase de ralentissement
      appel_rafraich_azim = true;                                           // Faire un appel au rafraîchissement de l'affichage
      digitalWrite(Relai_Frein,HIGH);                                       // Relâcher le relai du frein (enclencher le frein)
      appel_ralentissement = false;                                         // Terminer l'appel au ralentissement
      appel_arret = true;                                                   // Faire un appel à l'arrêt complet
      if (!alarme_active) tft.fillRect(20, 120, 130, 40, ILI9341_BLACK);    // s'il n'y a pas d'alarme active, effacer le champ de texte
      orient_cible = -1;                                                    // Effacer l'orientation cible
    } 
  }

  // Gestion de l'éteignement de l'affichage
  if ((millis() - debut_compte_affich_millis) >= 3600000)                   // Minuterie d'éteignement de l'affichage a atteint 1 heure?
  {
    digitalWrite(control_eclair_affich, LOW);                               // Oui, Éteindre l'affichage             
    debut_compte_affich_millis = millis();                                  // Réinitialiser la minuterie d'éteignement de l'affichage
  }
}


// Fonction perpétuelle de lecture de l'orientation instantanée. Roule sur le deuxième coeur du Pico.
void loop1()
{
  unsigned int curseur = 0;
  unsigned int maxval = 0;
  unsigned int minval = 0;
  for (int i=0; i<num_echant_pot; i++)                                          // Boucle de lecture du potentiomètre pour fins de moyenne
  {
    curseur += analogRead(Pot_Curseur);                                         // Faire la somme des lectures ADC du curseur
    maxval += analogRead(Pot_Max);                                              // Faire la somme des lectures ADC de la tension maximale du potard
    minval += analogRead(Pot_Min);                                              // Faire la somme des lectures ADC de la tension minimale du potard
    delay(1);                                                                   // Délai de 1 ms
  }
  curseur = round(float(curseur) / num_echant_pot);                             // Calcul de la moyenne des lectures ADC du curseur
  maxval = round(float(maxval) / num_echant_pot);                               // Calcul de la moyenne des lectures ADC de la tension maximale
  minval = round(float(minval) / num_echant_pot);                               // Calcul de la moyenne des lectures ADC de la tension minimale
  orient_instant = 360 - round((float(curseur) / (maxval - minval)) * 360);     // Calcul de la position instantanée. Attention, le potentiomètre du rotor est inversé!
  if (orient_instant > 360) orient_instant = 360;                               // Ne pas dépasser 360 degrés
  if (orient_instant < 0) orient_instant = 0;                                   // Ne pas permettre une orientation négative
  orient_dispo = true;                                                          // Aviser de la disponibilité d'une orientation instantanée
}
